@@ -35,17 +35,45 @@ class DarcsChangeset(Changeset):
         """
 
         super(DarcsChangeset, self).__init__(revision, date, author, log, entries=None, **other)
+        self.darcs_hash = other.get('darcs_hash')
         if entries is not None:
             for e in entries:
                 self.addEntry(e, revision)
 
+    def __eq__(self, other):
+        equal = (self.revision == other.revision and
+                 self.date == other.date and
+                 self.author == other.author)
+        if equal:
+            other_hash = getattr(other, 'darcs_hash', None)
+            if (self.darcs_hash is not None and
+                other_hash is not None and
+                self.darcs_hash != other_hash):
+                equal = False
+        return equal
+
+    def __ne__(self, other):
+        different = (self.revision <> other.revision or
+                     self.date <> other.date or
+                     self.author <> other.author)
+        if not different:
+            other_hash = getattr(other, 'darcs_hash', None)
+            if (self.darcs_hash is not None and
+                other_hash is not None and
+                self.darcs_hash != other_hash):
+                different = True
+        return different
+
     def addEntry(self, entry, revision):
         """
         Fixup darcs idiosyncrasies:
 
         - collapse "add A; rename A B" into "add B"
+        - collapse "rename A B; add B" into "add B"
         - annihilate "add A; remove A"
         - collapse "rename A B; remove B" into "remove A"
+        - collapse "rename A B; rename B C" into "rename A C"
+        - collapse "add A; edit A" into "add A"
         """
 
         # This should not happen, since the parser feeds us an already built
@@ -68,17 +96,39 @@ class DarcsChangeset(Changeset):
             # So, if there's a rename of this entry there, change that
             # to an addition instead, and don't insert any other entry
 
-            dirname = entry.name+'/' # darcs hopefully use forward slashes also under win
+            # darcs hopefully use forward slashes also under win
+            dirname = entry.name+'/'
 
             for i,e in enumerate(self.entries):
-                if e.action_kind == e.RENAMED and e.old_name == entry.name:
-                    # Luckily enough (since removes are the first entries
-                    # in the list, that is) by anticipating the add we
-                    # cure also the case below, when addition follows
-                    # edit.
-                    e.action_kind = e.ADDED
-                    e.old_name = None
-                    return e
+                if e.action_kind == e.RENAMED:
+                    if e.old_name == entry.name:
+                        # Unfortunately we have to check if the order if
+                        # messed up, in that case we should not do anything.
+                        # Example: mv a a2; mkdir a; mv a2 a/b
+                        skip = False
+                        for j in self.entries:
+                            if j.action_kind == j.RENAMED and j.name.startswith(dirname):
+                                skip = True
+                                break
+                        # Luckily enough (since removes are the first entries
+                        # in the list, that is) by anticipating the add we
+                        # cure also the case below, when addition follows
+                        # edit.
+                        if not skip:
+                            e.action_kind = e.ADDED
+                            e.old_name = None
+                            e.is_directory = entry.is_directory
+
+                            # Collapse "add A; edit A" into "add A"
+                            for j,oe in enumerate(self.entries):
+                                if oe.action_kind == e.UPDATED and e.name == oe.name:
+                                    del self.entries[j]
+
+                            return e
+
+                    # The "rename A B; add B" into "add B"
+                    if e.name == entry.name:
+                        del self.entries[i]
 
                 # Assert also that add_dir events must preceeds any
                 # add_file and ren_file that have that dir as target,
@@ -111,14 +161,34 @@ class DarcsChangeset(Changeset):
                     e.action_kind = e.DELETED
                     e.name = e.old_name
                     e.old_name = None
+                    e.is_directory = entry.is_directory
                     return e
                 elif e.action_kind == e.ADDED and e.name == entry.name:
                     del self.entries[i]
                     return None
 
+        # The "rename A B; rename B C" to "rename A C" part
+        elif entry.action_kind == entry.RENAMED:
+            # Adjust previous renames
+            olddirname = entry.old_name+'/'
+            for e in self.entries:
+                if e.action_kind == e.RENAMED and e.name.startswith(olddirname):
+                    e.name = entry.name + '/' + e.name[len(olddirname):]
+
+            for e in self.entries:
+                if e.action_kind == e.RENAMED and e.name == entry.old_name:
+                    e.name = entry.name
+                    return e
+
+            # The "rename A B; add B" into "add B", part two
+            for i,e in enumerate(self.entries):
+                if e.action_kind == e.ADDED and e.name == entry.name:
+                    return None
+
         # Ok, it must be either an edit or a rename: the former goes
         # obviously to the end, and since the latter, as said, come
         # in very early, appending is just good.
+
         self.entries.append(entry)
         return entry
 
@@ -189,6 +259,7 @@ def changesets_from_darcschanges_unsafe(changes, unidiff=False, repodir=None,
                 self.current['comment'] = ''
                 self.current['hash'] = attributes['hash']
                 self.current['entries'] = []
+                self.inverted = (attributes['inverted'] == "True")
             elif name in ['name', 'comment', 'add_file', 'add_directory',
                           'modify_file', 'remove_file', 'remove_directory']:
                 self.current_field = []
@@ -203,8 +274,8 @@ def changesets_from_darcschanges_unsafe(changes, unidiff=False, repodir=None,
                                       self.current['author'],
                                       self.current['comment'],
                                       self.current['entries'],
-                                      tags=self.current.get('tags',[]))
-                cset.darcs_hash = self.current['hash']
+                                      tags=self.current.get('tags',[]),
+                                      darcs_hash=self.current['hash'])
                 if self.darcsdiff:
                     cset.unidiff = self.darcsdiff.execute(TZ='UTC',
                         stdout=PIPE, patchname=cset.revision)[0].read()
@@ -223,14 +294,21 @@ def changesets_from_darcschanges_unsafe(changes, unidiff=False, repodir=None,
                 self.current['entries'].append(entry)
             elif name in ['add_file', 'add_directory', 'modify_file',
                           'remove_file', 'remove_directory']:
-                entry = ChangesetEntry(''.join(self.current_field).strip())
+                current_field = ''.join(self.current_field).strip()
+                if self.inverted:
+                    # the filenames in file modifications are outdated
+                    # if there are renames
+                    for i in self.current['entries']:
+                        if i.action_kind == i.RENAMED and current_field.startswith(i.old_name):
+                            current_field = current_field.replace(i.old_name, i.name)
+                entry = ChangesetEntry(current_field)
                 entry.action_kind = { 'add_file': entry.ADDED,
                                       'add_directory': entry.ADDED,
                                       'modify_file': entry.UPDATED,
                                       'remove_file': entry.DELETED,
                                       'remove_directory': entry.DELETED
                                     }[name]
-
+                entry.is_directory = name.endswith('directory')
                 self.current['entries'].append(entry)
 
         def characters(self, data):
@@ -272,18 +350,62 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
         Do the actual work of fetching the upstream changeset.
         """
 
-        cmd = self.repository.command("pull", "--dry-run")
-        pull = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-        output = pull.execute(self.repository.repository,
-                              stdout=PIPE, stderr=STDOUT, TZ='UTC0')[0]
+        # Use the newer pull --xml-output, if possible
+        use_xml = False
+        if self.repository.darcs_version.startswith('2'):
+            cmd = self.repository.command("pull", "--dry-run", "--xml-output")
+            pull = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+            output = pull.execute(self.repository.repository,
+                                  stdout=PIPE, stderr=STDOUT, TZ='UTC0')[0]
+            # pull --xml-output was introduced *after* 2.0.0
+            if pull.exit_status:
+                errormsg = output.read()
+                if "unrecognized option `--xml-output'" in errormsg:
+                    self.log.warning('Using darcs 1.0 non-XML parser: it may fail '
+                                     'on patches recorded before november 2003! '
+                                     'I would suggest of upgrading to latest darcs 2.0 '
+                                     '(later than 2.0+233).')
+                    # No way, fall back to old behaviour, that will possibly fail,
+                    # on patches recorded before 2003-11-01... :-|
+                else:
+                    raise GetUpstreamChangesetsFailure(
+                        "%s returned status %d saying\n%s" %
+                        (str(pull), pull.exit_status, errormsg))
+            else:
+                use_xml = True
 
-        if pull.exit_status:
-            raise GetUpstreamChangesetsFailure(
-                "%s returned status %d saying\n%s" %
-                (str(pull), pull.exit_status, output.read()))
+        if not use_xml:
+            cmd = self.repository.command("pull", "--dry-run")
+            pull = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+            output = pull.execute(self.repository.repository,
+                                  stdout=PIPE, stderr=STDOUT, TZ='UTC0')[0]
 
-        return self._parseDarcsPull(output)
+            if pull.exit_status:
+                raise GetUpstreamChangesetsFailure(
+                    "%s returned status %d saying\n%s" %
+                    (str(pull), pull.exit_status, output.read()))
 
+            return self._parseDarcsPull(output)
+        else:
+            from cStringIO import StringIO
+
+            # My initial implementation of --xml-output on darcs pull
+            # wasn't perfect, as it was printing useless verbosity before
+            # and after the actual xml. Around 2.0.0+275 I removed that...
+
+            line = output.readline() # Would pull from "/home/lele/wip/darcs-2.0"...
+            if line.startswith('Would pull from '):
+                # Skip the first two lines and drop the last two as well
+                output.readline() # Would pull the following changes:
+                xml = StringIO(''.join(output.readlines()[:-2]))
+                xml.seek(0)
+            else:
+                output.seek(0)
+                xml = output
+
+            badchars = self.repository.replace_badchars
+
+            return changesets_from_darcschanges(xml, replace_badchars=badchars)
 
     def _parseDarcsPull(self, output):
         """Process 'darcs pull' output to Changesets.
@@ -291,7 +413,6 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
         from datetime import datetime
         from time import strptime
         from sha import new
-        from vcpx.changes import Changeset
 
         l = output.readline()
         while l and not (l.startswith('Would pull the following changes:') or
@@ -312,7 +433,11 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                 pieces = fsep.split(l.rstrip(), 8)
                 assert len(pieces)>=7, "Cannot parse %r as a patch timestamp" % l
                 date = ' '.join(pieces[:8])
-                author = pieces[8]
+                try:
+                    author = pieces[8]
+                except IndexError, s:
+                    # darcs allows patches with empty author
+                    author = ""
                 y,m,d,hh,mm,ss,d1,d2,d3 = strptime(date, "%a %b %d %H %M %S %Z %Y")
                 date = datetime(y,m,d,hh,mm,ss,0,UTC)
                 l = output.readline().rstrip()
@@ -332,7 +457,7 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                     changelog.append(l[2:-1])
                     l = output.readline()
 
-                cset = Changeset(name, date, author, '\n'.join(changelog))
+                cset = DarcsChangeset(name, date, author, '\n'.join(changelog))
                 compactdate = date.strftime("%Y%m%d%H%M%S")
                 if name.startswith('UNDO: '):
                     name = name[6:]
@@ -414,15 +539,21 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                 conflicts.extend(files)
             line = output.readline()
 
+        # Complete the changeset with its entries
+
         cmd = self.repository.command("changes", selector, revtag,
                                       "--xml-output", "--summ")
         changes = ExternalCommand(cwd=self.repository.basedir, command=cmd)
         last = changesets_from_darcschanges(changes.execute(stdout=PIPE)[0],
                                             replace_badchars=self.repository.replace_badchars)
         try:
-            changeset.entries.extend(last.next().entries)
+            entries = last.next().entries
         except StopIteration:
-            pass
+            entries = None
+
+        if entries:
+            for e in entries:
+                changeset.addEntry(e, changeset.revision)
 
         return conflicts
 
@@ -435,11 +566,23 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
         be none of them in tailor context.
         """
 
+        from os import walk, unlink
+        from os.path import join
+        from re import compile
+
         self.log.info("Reverting changes to %s, to solve the conflict",
                       ' '.join(conflict))
         cmd = self.repository.command("revert", "--all")
         revert = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-        revert.execute(conflict)
+        revert.execute(conflict, input="\n")
+
+        # Remove also the backups made by darcs
+        bckre = compile('-darcs-backup[0-9]+$')
+        for root, dirs, files in walk(self.repository.basedir):
+            backups = [f for f in files if bckre.search(f)]
+            for bck in backups:
+                self.log.debug("Removing backup file %r in %r", bck, root)
+                unlink(join(root, bck))
 
     def _checkoutUpstreamRevision(self, revision):
         """
@@ -471,7 +614,11 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                          output and output.read() or ''))
 
                 csets = changesets_from_darcschanges(output, replace_badchars=self.repository.replace_badchars)
-                changeset = csets.next()
+                try:
+                    changeset = csets.next()
+                except StopIteration:
+                    # No changesets, no party!
+                    return None
 
                 revision = 'hash %s' % changeset.darcs_hash
             else:
@@ -479,7 +626,9 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
         else:
             initial = False
 
-        if self.repository.subdir == '.' or exists(self.repository.basedir):
+        # Darcs 2.0 fails with "darcs get --to-match", see issue885
+        darcs2 = self.repository.darcs_version.startswith('2')
+        if darcs2 or self.repository.subdir == '.' or exists(self.repository.basedir):
             # This is currently *very* slow, compared to the darcs get
             # below!
             if not exists(join(self.repository.basedir, '_darcs')):
@@ -532,6 +681,10 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                 "%s returned status %d saying\n%s" %
                 (str(changes), changes.exit_status, output.read()))
 
-        last = changesets_from_darcschanges(output, replace_badchars=self.repository.replace_badchars)
+        try:
+            last = changesets_from_darcschanges(
+                output, replace_badchars=self.repository.replace_badchars).next()
+        except StopIteration:
+            last = None
 
-        return last.next()
+        return last
