@@ -11,11 +11,13 @@ This module contains supporting classes for Subversion.
 
 __docformat__ = 'reStructuredText'
 
-from vcpx.repository import Repository
-from vcpx.shwrap import ExternalCommand, PIPE, ReopenableNamedTemporaryFile
-from vcpx.source import UpdatableSourceWorkingDir, ChangesetApplicationFailure
-from vcpx.target import SynchronizableTargetWorkingDir, TargetInitializationFailure
+from vcpx.changes import ChangesetEntry
 from vcpx.config import ConfigurationError
+from vcpx.repository import Repository
+from vcpx.shwrap import ExternalCommand, PIPE, STDOUT, ReopenableNamedTemporaryFile
+from vcpx.source import UpdatableSourceWorkingDir, ChangesetApplicationFailure
+from vcpx.target import SynchronizableTargetWorkingDir, TargetInitializationFailure, \
+                        PostCommitCheckFailure
 from vcpx.tzinfo import UTC
 
 
@@ -134,7 +136,10 @@ class SvnRepository(Repository):
                                                   "repository" %
                                                   self.repository)
 
-            repodir = self.repository[7:]
+            if platform != 'win32':
+                repodir = self.repository[7:]
+            else:
+                repodir = self.repository[8:]
             cmd = self.command("create", "--fs-type", "fsfs", svnadmin=True)
             svnadmin = ExternalCommand(command=cmd)
             svnadmin.execute(repodir)
@@ -149,7 +154,10 @@ class SvnRepository(Repository):
                                  "has the 'pre-revprop-change' hook active, needed "
                                  "by 'use-propset=True'. Assuming it does...")
             else:
-                repodir = self.repository[7:]
+                if platform != 'win32':
+                    repodir = self.repository[7:]
+                else:
+                    repodir = self.repository[8:]
                 hookname = join(repodir, 'hooks', 'pre-revprop-change')
                 if platform == 'win32':
                     hookname += '.bat'
@@ -359,7 +367,6 @@ def changesets_from_svnlog(log, repository, chunksize=2**15):
                 entrypath = get_entry_from_path(path)
                 if entrypath:
                     entry = ChangesetEntry(entrypath)
-
                     if type(self.current_path_action) == type( () ):
                         self.copies.append(entry.name)
                         old = get_entry_from_path(self.current_path_action[1])
@@ -434,8 +441,16 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         return changesets_from_svnlog(log, self.repository)
 
     def _applyChangeset(self, changeset):
+        from os import walk
+        from os.path import join, isdir
         from time import sleep
 
+        # Complete changeset information, determining the is_directory
+        # flag of the removed entries, before updating to the given revision
+        for entry in changeset.entries:
+            if entry.action_kind == entry.DELETED:
+                entry.is_directory = isdir(join(self.repository.basedir, entry.name))
+
         cmd = self.repository.command("update")
         if self.repository.ignore_externals:
             cmd.append("--ignore-externals")
@@ -468,6 +483,37 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
                        ','.join([e.name for e in changeset.entries]),
                        changeset.revision)
 
+        # Complete changeset information, determining the is_directory
+        # flag of the added entries
+        implicitly_added_entries = []
+        known_added_entries = set()
+        for entry in changeset.entries:
+            if entry.action_kind == entry.ADDED:
+                known_added_entries.add(entry.name)
+                fullname = join(self.repository.basedir, entry.name)
+                entry.is_directory = isdir(fullname)
+                # If it is a directory, extend the entries of the
+                # changeset with all its contents, if not already there.
+                if entry.is_directory:
+                    for root, subdirs, files in walk(fullname):
+                        if '.svn' in subdirs:
+                            subdirs.remove('.svn')
+                        for f in files:
+                            name = join(root, f)[len(self.repository.basedir)+1:]
+                            newe = ChangesetEntry(name)
+                            newe.action_kind = newe.ADDED
+                            implicitly_added_entries.append(newe)
+                        for d in subdirs:
+                            name = join(root, d)[len(self.repository.basedir)+1:]
+                            newe = ChangesetEntry(name)
+                            newe.action_kind = newe.ADDED
+                            newe.is_directory = True
+                            implicitly_added_entries.append(newe)
+
+        for e in implicitly_added_entries:
+            if not e.name in known_added_entries:
+                changeset.entries.append(e)
+
         result = []
         for line in out:
             if len(line)>2 and line[0] == 'C' and line[1] == ' ':
@@ -516,14 +562,16 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
                     reporoot = reporoot[:reporoot.rfind('/')]
 
         if lastok is None:
-            raise ConfigurationError("%r is not the root of a svn repository." %
+            raise ConfigurationError('%r is not the root of a svn repository. If '
+                                     'you are sure it is indeed, you may try setting '
+                                     'the option "trust-root" to "True".' %
                                      self.repository.repository)
         elif lastok <> self.repository.repository:
             module = self.repository.repository[len(lastok):]
             module += self.repository.module
-            raise ConfigurationError("Non-root svn repository %r. "
-                                     "Please specify that as 'repository=%s' "
-                                     "and 'module=%s'." %
+            raise ConfigurationError('Non-root svn repository %r. '
+                                     'Please specify that as "repository=%s" '
+                                     'and "module=%s".' %
                                      (self.repository.repository,
                                       lastok, module.rstrip('/')))
 
@@ -719,6 +767,19 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
 
         ExternalCommand(cwd=self.repository.basedir, command=cmd).execute()
 
+    def _postCommitCheck(self):
+        """
+        Assert that all the entries in the working dir are versioned.
+        """
+
+        cmd = self.repository.command("status")
+        whatsnew = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+        output = whatsnew.execute(stdout=PIPE, stderr=STDOUT)[0]
+        unknown = [l for l in output.readlines() if l.startswith('?')]
+        if unknown:
+            raise PostCommitCheckFailure(
+                "Changes left in working dir after commit:\n%s" % ''.join(unknown))
+
     def _removePathnames(self, names):
         """
         Remove some filesystem objects.
@@ -795,6 +856,7 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         """
 
         from os.path import join, exists
+        from vcpx.dualwd import IGNORED_METADIRS
 
         if not self.repository.repository or exists(join(self.repository.basedir, self.repository.METADIR)):
             return
@@ -807,6 +869,20 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
         svnco.execute("%s%s" % (self.repository.repository,
                                 self.repository.module), self.repository.basedir)
 
+        ignore = [md for md in IGNORED_METADIRS]
+
+        if self.logfile.startswith(self.repository.basedir):
+            ignore.append(self.logfile[len(self.repository.basedir)+1:])
+        if self.state_file.filename.startswith(self.repository.basedir):
+            sfrelname = self.state_file.filename[len(self.repository.basedir)+1:]
+            ignore.append(sfrelname)
+            ignore.append(sfrelname+'.old')
+            ignore.append(sfrelname+'.journal')
+
+        cmd = self.repository.command("propset", "%(propname)s", "--quiet")
+        pset = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+        pset.execute('\n'.join(ignore), '.', propname='svn:ignore')
+
     def _initializeWorkingDir(self):
         """
         Add the given directory to an already existing svn working tree.
