@@ -36,6 +36,10 @@ class ChangesetReplayFailure(TailorException):
     "Failure replaying the changeset on the target system"
 
 
+class PostCommitCheckFailure(TailorException):
+    "Most probably a tailor bug, not everything has been committed."
+
+
 class SynchronizableTargetWorkingDir(WorkingDir):
     """
     This is an abstract working dir usable as a *shadow* of another
@@ -97,6 +101,19 @@ class SynchronizableTargetWorkingDir(WorkingDir):
             changelog = changeset.log
         return patchname, changelog
 
+    def _prepareToReplayChangeset(self, changeset):
+        """
+        This is called **before** fetching and applying the source
+        changeset. This implementation does nothing more than
+        returning True. Subclasses may override it, for example to
+        preexecute some entries such as moves.
+
+        Returning False the changeset won't be applied and the
+        process will stop.
+        """
+
+        return True
+
     def replayChangeset(self, changeset):
         """
         Do whatever is needed to replay the changes under the target
@@ -127,6 +144,8 @@ class SynchronizableTargetWorkingDir(WorkingDir):
             if changeset.tags:
                 for tag in changeset.tags:
                     self._tag(tag, changeset.date, changeset.author)
+            if self.repository.post_commit_check:
+                self._postCommitCheck()
         finally:
             signal(SIGINT, previous)
 
@@ -284,14 +303,13 @@ class SynchronizableTargetWorkingDir(WorkingDir):
 
     def _replayChangeset(self, changeset):
         """
-        Replicate the actions performed by the changeset on the tree of
-        files.
+        Replay each entry of the changeset, that is execute the action associated
+        to each kind of change for each entry, possibly grouping consecutive entries
+        of the same kind.
         """
 
-        from os.path import join, isdir
         from changes import ChangesetEntry
 
-        added = []
         actions = { ChangesetEntry.ADDED: self._addEntries,
                     ChangesetEntry.DELETED: self._removeEntries,
                     ChangesetEntry.RENAMED: self._renameEntries,
@@ -312,23 +330,11 @@ class SynchronizableTargetWorkingDir(WorkingDir):
                     action(group)
                 group = [e]
                 last = e
-            if e.action_kind == e.ADDED:
-                added.append(e)
         if group:
             action = actions.get(group[0].action_kind)
             if action is not None:
                 action(group)
 
-        # Finally, deal with "copied" directories. The simple way is
-        # executing an _addSubtree on each of them, evenif this may
-        # cause "warnings" on items just moved/added above...
-
-        while added:
-            subdir = added.pop(0).name
-            if isdir(join(self.repository.basedir, subdir)):
-                self._addSubtree(subdir)
-                added = [e for e in added if not e.name.startswith(subdir)]
-
     def _addEntries(self, entries):
         """
         Add a sequence of entries
@@ -395,6 +401,13 @@ class SynchronizableTargetWorkingDir(WorkingDir):
 
         raise TailorBug("%s should override this method!" % self.__class__)
 
+    def _postCommitCheck(self):
+        """
+        Perform any safety-belt check to assert everything's ok. This
+        implementation does nothing, subclasses should reimplement the
+        method.
+        """
+
     def _removeEntries(self, entries):
         """
         Remove a sequence of entries.
@@ -495,7 +508,10 @@ class SynchronizableTargetWorkingDir(WorkingDir):
 
                 if renamed:
                     if self.shared_basedirs:
-                        rename(absold + '-TAILOR-HACKED-TEMP-NAME', absold)
+                        # it's possible that the target already handled
+                        # this
+                        if exists(absold + '-TAILOR-HACKED-TEMP-NAME'):
+                            rename(absold + '-TAILOR-HACKED-TEMP-NAME', absold)
                     else:
 
                         # before rsync      after rsync      the HACK            after "svn mv"           result
